/* **************************************************************************
    MP3-IndexMaker v0.1 - id3.c
    Copyright (c)1998 by Brian James Chapman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
    
    Support the Free Software Foundation at http://www.fsf.org
*************************************************************************** */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <errno.h>
#include "id3.h"


/* remove trailing white space, some Win95 programs (ie: AudioGrabber) 
   pad remaing feild area with spaces rather than '\0'  */
int rmtws(char *s) {
	char *p;
	p = s + strlen(s) -1;
	if (*p != 0x20)
		return(-1);
	while (*p == 0x20)
		p--;
	p++;
	*p = '\0';
	return(0);
}
	
/* translate genre value into a more human identifiable meaning */
char *get_genre_name(char g[])
{
	static char *name[] = {
		"Blues","Classic Rock","Country","Dance","Disco","Funk","Grunge",
		"Hip-Hop","Jazz","Metal","New Age","Oldies","Other","Pop","R&B","Rap","Reggae",
		"Rock","Techno","Industrial","Alternative","Ska","Death Metal","Pranks",
		"Soundtrack","Euro-Techno","Ambient","Trip-Hop","Vocal","Jazz+Funk","Fusion",
		"Trance","Classical","Instrumental","Acid","House","Game","Sound Clip",
		"Gospel","Noise","AlternRock","Bass","Soul","Punk","Space","Meditative",
		"Instrumental Pop","Instrumental Rock","Ethnic","Gothic","Darkwave",
		"Techno-Industrial","Electronic","Pop-Folk","Eurodance","Dream","Southern Rock",
		"Comedy","Cult","Gangsta","Top 40","Christian Rap","Pop/Funk","Jungle","Native American",
		"Cabaret","New Wave","Psychadelic","Rave","Showtunes","Trailer","Lo-Fi","Tribal",
		"Acid Punk","Acid Jazz","Polka","Retro","Musical","Rock & Roll","Hard Rock","Folk",
		"Folk/Rock","National Folk","Swing","Bebob","Latin","Revival","Celtic","Bluegrass",
		"Avantgarde","Gothic Rock","Progressive Rock","Psychedelic Rock","Symphonic Rock",
		"Slow Rock","Big Band","Chorus","Easy Listening","Acoustic","Humour","Speech","Chanson",
		"Opera","Chamber Music","Sonata","Symphony","Booty Bass","Primus","Porn Groove",
		"Satire","Slow Jam","Club","Tango","Samba","Folklore","Undefined"
	};
	
	int c = (int)g[0];

	return((c < 1 || c > 115) ? name[115] : name[c]);
}

int fgetstr(char *p, int bytes, FILE *f)
{
	int i;
	for(i=0; i < bytes; i++) {
		*p = fgetc(f);
		p++;
	}
	*p = '\0';
	return(0);
}

/* returns: 0 if no tag found, 1 if found, -1 if error */
int get_id3info(FILE *mp3, ID3 *id3, int flag)
{
	if (fseek(mp3, -128, SEEK_END) != 0 )
		return(-1);
	
	fgetstr(id3->tag,ID3_MAX_TAG,mp3);
	if (strcmp(id3->tag,"TAG"))
		return(0);
	fgetstr(id3->title,ID3_MAX_TITLE,mp3);
	rmtws(id3->title);
	fgetstr(id3->artist,ID3_MAX_ARTIST,mp3);
	rmtws(id3->artist);
	fgetstr(id3->album,ID3_MAX_ALBUM,mp3);
	rmtws(id3->album);
	fgetstr(id3->year,ID3_MAX_YEAR,mp3);
	fgetstr(id3->comment,ID3_MAX_COMMENT,mp3);
	rmtws(id3->comment);
	fgetstr(id3->genre,ID3_MAX_GENRE,mp3);

	return(1);
}


#ifdef TEST

int main(int argc, char *argv[]) 
{

	ID3 id3;
	FILE *mp3;
	int ret;
	
	if (argc != 2) {
		printf("Usage:  %s  <mp3file>\n",argv[0]);
		exit(1);
	}
	
	memset(&id3, 0, sizeof(id3));

	if ((mp3=fopen(argv[1],"rb")) == NULL) {
		/* perror(argv[1]); */
		printf("error opening mp3 file\n");
		exit(1);
	}

	switch ( get_id3info(mp3, &id3, 1) ) {
		case 0:
			printf("File does not contain ID3 tag.\n");
			exit(1);
			break;
		case -1:
			printf("Error while scanning for ID3 tag info\n");
			exit(1);
			break;
		default:
			break;
	}

	fclose(mp3);
	
	printf("\n");
	printf(" ID3-Tag (version 1.0) Information\n\n");
	printf("      tag:%s:\n"
	       "    title:%s:\n"
           "   artist:%s:\n"
	       "    album:%s:\n"
	       "     year:%s:\n"
           "  comment:%s:\n"
	       "    genre:%s:\n",
	       id3.tag,id3.title,id3.artist,id3.album,
	       id3.year,id3.comment,get_genre_name(id3.genre));
	printf("\n");
	
	return(0);
}

#endif