/* **************************************************************************
    MP3-IndexMaker v0.1 - mkindex.c
    Copyright (c)1998 by Brian James Chapman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
    
    Support the Free Software Foundation at http://www.fsf.org
*************************************************************************** */

#include <unistd.h>
#include <stdio.h>
#include <dirent.h>
#include <limits.h>
#include <string.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include "id3.h"
#include "getopt.h"

#define IDEPTH 0
#define TEXT 0
#define HTML 1
#define CSV  2

int verbose=0, format = 0, recursion = 1;
char *outfile = "index";
char *arbdir = NULL, *program;
int DirCnt=0, FileCnt=0, Mp3Cnt=0, ErrCnt=0;

void version(void) {
	printf("MP3-IndexMaker v0.1\n");
	exit(0);
	return;
}

void usage(int level)
{
	printf( 
	"Usage: %s [options]\n"
	"Options:\n"
	"  -h, --help              Shows this info.\n"
	"  -v, --verbose           Displays stuff.\n"
	"  -V, --version           Shows version.\n"
	"  -n, --no-recursion      Do not recurse into subdirectories.\n"
	"  -f FORMAT, --output-format=FORMAT\n"
	"                          Output format. Supported formats: HTML,\n"
	"                          CSV, or TEXT (default).\n"
	"  -o FILE, --output-file=FILE\n"
	"                          Specify output filename (defaults to index\n"
	"                          index.html, or index.csv)\n"
	"  -C DIRECTORY, --directory=DIRECTORY\n"
	"                          Specify directory to start in.(CWD default)\n"
	,program);
	exit(level);
	return;
}

void fprintdir(FILE *of, char *dir, int depth)
{
	DIR *dp;
	ID3 id3;
	FILE *mp3;
	struct dirent *entry;
	struct stat statbuf;
	double fsize;
	char *p;
	
	if (((dp = opendir(dir)) == NULL)) {
		fprintf(stderr,"cannot open directory: %s\n", dir);
		return;
	}
	chdir(dir);
	while((entry = readdir(dp)) != NULL) {
		stat(entry->d_name,&statbuf);
		if(S_ISDIR(statbuf.st_mode)) { // check to see if its a directory
			if(strcmp(".",entry->d_name) == 0 || strcmp("..",entry->d_name) == 0 )
			   continue; // its a dir but skip the . and .. stuff
			/* if (format == CSV)
				fprintf(of,"\042DIR\042,\042%s\042\n",entry->d_name);
			else */
			fprintf(of,"\n%*s%s/\n",depth," ",entry->d_name); // print directory name
			DirCnt++;
			fprintdir(of, entry->d_name,depth + IDEPTH);            // and recurse into it.
		} else {
			fsize = (((double)(statbuf).st_size) / 1024) / 1024;
			p = entry->d_name;
			p = p + strlen(entry->d_name) - 4;
			if (strcasecmp(p,".mp3") == 0) { // record only files ending in .mp3
				if (strlen(entry->d_name) > 20)
					fprintf(of,"%*s%-17.17s...  %2.2lf MB",depth," ",entry->d_name,fsize);
				else
					fprintf(of,"%*s%-20.20s  %2.2lf MB",depth," ",entry->d_name,fsize); // print file name
				memset(&id3, 0, sizeof(id3));
				if ( (mp3=fopen(entry->d_name,"rb")) != NULL ) {
					if (get_id3info(mp3, &id3, 1)) {
						Mp3Cnt++;
						fprintf(of,"  %-30s   %-30s  %-30s %-4s  %-30s",id3.title,id3.album,id3.artist,id3.year,get_genre_name(id3.genre));
					}
				} else {
					printf("error opening mp3 file: %s\n",entry->d_name);
					ErrCnt++;
				}
				fclose(mp3);
				fprintf(of,"\n");
				FileCnt++;
			}
		}
	}
	chdir("..");
	closedir(dp);
}


int main(int argc, char **argv)
{
	FILE *of;
	int  i,c,namechange = 0,digit_optind = 0;	
	char *wkdir = (char *)malloc(PATH_MAX);
	char **q;

	/* find program name - special thanks to the GNU make sources */
	if (argv[0] == 0)
		argv[0] = "";
	if (argv[0][0] == '\0')
		program = "mkindex";
	else {
		program = rindex(argv[0], '/');
#ifdef __MSDOS__
		if (program == 0)
			program = rindex(argv[0], '\\');
		else {
	  	/* Some weird environments might pass us argv[0] with
	     	   both kinds of slashes; we must find the rightmost.  */
		char *q = rindex (argv[0], '\\');
		if (q && q > program)
			program = q;
		}
	if (program == 0 && argv[0][1] == ':')
		program = argv[0] + 1;
#endif
		if (program == 0)
			program = argv[0];
		else
			++program;
	}
	
	strcpy(argv[0],program);
	
	while (1) {
		int this_option_optind = optind ? optind : 1;
		int option_index = 0;
		static struct option long_options[] = {
			{"help", 0, 0, 'h'},
			{"version", 0, 0, 'V'},
			{"no-recursion", 0, 0, 'n'},
			{"verbose", 0, 0, 'v'},
			{"output-format", 1, 0, 'f'},
			{"output-file", 1, 0, 'o'},
			{"directory", 1, 0, 'C'},
			{0, 0, 0, 0}
		};

		c = getopt_long (argc, argv, "hVnvf:o:C:", long_options, &option_index);
		if (c == -1)
			break;

		switch (c) {
		case 'h':
			usage(0);
			break;
		case 'V':
			version();
			break;
		case 'n':
			recursion = 0;
			break;
		case 'v':
			verbose++;
			break;
		case 'f':
			if (strcasecmp(optarg,"text") == 0) {
				format = TEXT;
			/* } else if (strcasecmp(optarg,"html") == 0) {
				format = HTML;
				if (!namechange)
					strcpy(outfile,"index.html");
			} else if (strcasecmp(optarg,"csv") == 0) {
				format = CSV;
				if (!namechange)
					strcpy(outfile,"index.csv"); */
			} else {
				printf("Format not supported: %s\n",optarg);
				exit(1);
			}
			break;
		case 'o':
			namechange = 1;
			strcpy(outfile,optarg);
			break;
		case 'C':
			arbdir = optarg;
			break;
		case '?':
			break;
		default:
			printf ("?? getopt returned character code 0%o ??\n", c);
			exit(1);
			break;
		}
	}

         if (optind < argc)
           {
             printf ("Unrecgonized argument(s): ");
             while (optind < argc)
             	printf ("%s ", argv[optind++]);
             printf ("\n");
             usage(1);
           }
	
	if(getcwd(wkdir, PATH_MAX) == NULL) {
		fprintf(stderr,"Could not obtain working directory!\n");
		exit(1);
	}

	if ((of = fopen(outfile,"wt")) == NULL) {
		fprintf(stderr,"cannot open file for output\n");
		return;
	}

	fprintdir(of, arbdir ? arbdir : wkdir, 0);
	fclose(of);

	if (verbose) {
		printf("Scanned %i directories containing %i MP3 files.\n",DirCnt,FileCnt);
		printf("%i of %i files contained ID3-Tag information.\n",Mp3Cnt,FileCnt-ErrCnt);
		if (ErrCnt)
			printf("%i file errors encountered.\n",ErrCnt);
	}
	
#ifdef DEBUG
	printf("\n");
	printf(	   "verbose     : %i\n"
		   "outfile     : %s\n"
		   "wkdir       : %s\n"		   
		   "arbdir      : %s\n"
		   "format      : %i\n"
		   "recursion   : %i\n",
		   verbose,outfile,wkdir,arbdir,format,recursion);
#endif

	chdir(wkdir);
	free(wkdir);
	return 0;
}
